feature_info = {
    "crimes_pp": {
        "title": "Erfasste Straftaten",
        "label": "Straftaten / EW",
        "time_dep": True,
        "color": "gray_r"
    },
    "fraud_pp": {
        "title": "Betrugsfälle",
        "label": "Betrugsfälle / EW",
        "time_dep": True,
        "color": "gray_r"
    },
    "theft_pp": {
        "title": "Diebstähle",
        "label": "Diebstähle / EW",
        "time_dep": True,
        "color": "gray_r"
    },
    "violence_pp": {
        "title": "Gewaltkriminalität",
        "label": "Straftaten / EW",
        "time_dep": True,
        "color": "gray_r"
    },
    "drug_pp": {
        "title": "Rauschgiftdelikte",
        "label": "Fälle / EW",
        "time_dep": True,
        "color": "gray_r"
    },
    "income_pp": {
        "title": "Verfügbares Einkommen der privaten Haushalte",
        "label": "Euro / EW",
        "time_dep": True,
        "color": "gray"
    },
    "hh_avg": {
        "title": "Durchschnittliche Haushaltsgröße (2011)",
        "label": "Personen",
        "time_dep": False,
        "color": "gray_r"
    },
    "population": {
        "title": "Bevölkerung",
        "label": "EW",
        "time_dep": True,
        "color": "gray_r"
    },
    "population_density": {
        "title": "Bevölkerungsdichte",
        "label": "EW / qkm",
        "time_dep": True,
        "color": "gray_r"
    },
    "prop_abitur": {
        "title": "Anteil Schulabgänger/-innen mit allgem. Hochschulreife",
        "label": "%",
        "time_dep": True,
        "color": "gray_r"
    },
    "prop_no_haupt": {
        "title": "Anteil Schulabgänger/-innen ohne Hauptschulabschluss",
        "label": "%",
        "time_dep": True,
        "color": "gray_r"
    },
    "border_vic": {
        "title": "Entfernung zur Grenze",
        "label": "Km",
        "time_dep": False,
        "color": "gray_r"
    },
    "ew_vot_rel_alternative_fuer_deutschland_2019": {
        "title": "AFD-Stimmanteil, Europawahl 2019",
        "label": "%",
        "time_dep": False,
        "color": "blues"
    },
    "ew_vot_rel_sozialdemokratische_partei_deutschlands_2019": {
        "title": "SPD-Stimmanteil, Europawahl 2019",
        "label": "%",
        "time_dep": False,
        "color": "reds"
    },
    "ew_vot_rel_buendnis_90_die_gruenen_2019": {
        "title": "Grüne-Stimmanteil, Europawahl 2019",
        "label": "%",
        "time_dep": False,
        "color": "greens"
    },
    "ew_vot_rel_christlich_demokratische_union_deutschlands_2019": {
        "title": "CDU/CSU-Stimmanteil, Europawahl 2019",
        "label": "%",
        "time_dep": False,
        "color": "greys"
    },
    "ew_vot_rel_freie_demokratische_partei_2019": {
        "title": "FDP-Stimmanteil, Europawahl 2019",
        "label": "%",
        "time_dep": False,
        "color": ["white", "yellow"]
    },
    "ew_vot_rel_die_linke_2019": {
        "title": "Linke-Stimmanteil, Europawahl 2019",
        "label": "%",
        "time_dep": False,
        "color": "reds"
    },
    "ew_winner_2019": {
        "title": "Europa-Wahl 2019",
        "label": "Partei",
        "time_dep": False,
        "color": {
            "alternative_fuer_deutschland": "lightblue",
            'sozialdemokratische_partei_deutschlands': "red",
            'buendnis_90_die_gruenen': "lightgreen",
            'christlich_demokratische_union_deutschlands': "darkgray",
            "freie_demokratische_partei": "yellow",
            "die_linke": "red"
        }
    }
}
