from . import regions
from . import population
from . import license_plate
from . import income
from . import household
from . import database
from . import utils
from . import border_vicinity
from . import crime
from . import education
from . import election

db = database.DataBase()
